/* 44780 LCD Display Ansteuerung      	*/
/* zu definieren : 		*/
/* lcd_en_dir : Datenrichtungsregister fr Enable	*/
/* lcd_en_port : Port, an dem enable angeschlossen ist 		*/
/* lcd_en_bit : Bit-Nr der enable-Leitung an en_port		*/
/* lcd_rs_port : Port, an dem Data/Command angeschlossen ist
/* lcd_rs_dir : Datenrichtungsregister fr rs Port	*/
/* lcd_rs_bit : Bit-Nr der Data/Command an dc_port		*/
/* lcd_port : Port, an dem Datenleitungen angeschlossen sind, im Vier-Bit-Modus an Bits 4-7; 0-3 bleibt frei 	*/
/* lcd_dir : Richtungsregister fr Datenleitungen */

void lcd_data(void)
{
	lcd_rs_port |= (1 << lcd_rs_bit);
}

void lcd_comm(void)
{
	lcd_rs_port &= ~(1 << lcd_rs_bit);
}

void lcd_acht_write(unsigned char value)
{
	lcd_port = (value & 0xf0);
	lcd_en_port &= ~(1 << lcd_en_bit);
	_delay_us(100);
	lcd_en_port |= (1 << lcd_en_bit);
}

void lcd_vollacht_write(unsigned char value)
{
	lcd_port = value;
	lcd_en_port &= ~(1 << lcd_en_bit);
	_delay_us(100);
	lcd_en_port |= (1 << lcd_en_bit);
}

void lcd_write(unsigned char value)
{
	lcd_port &= 0x0f;
	lcd_port |= (value & 0xf0);
	lcd_en_port &= ~(1 << lcd_en_bit);
	_delay_us(100);
	lcd_en_port |= (1 << lcd_en_bit);

	lcd_port &= 0x0f;
	lcd_port |= ((value << 4) & 0xf0);
	lcd_en_port &= ~(1 << lcd_en_bit);
	_delay_us(100);
	lcd_en_port |= (1 << lcd_en_bit);
}
void lcd_putchar(unsigned char a)
{
	lcd_data();
	lcd_write(a);
}

void lcd_acht_putcomm(unsigned char a)
{
	lcd_comm();
	lcd_acht_write(a);
}

void lcd_putcomm(unsigned char a)
{
	lcd_comm();
	lcd_write(a);
}

void lcd_string(char *a)
{
	char *b;
	
	b = a;

	while(*b != 0)
	{
		lcd_putchar(*b);
		b++;
	}
}

void lcd_setupports(void)
{
	lcd_rs_dir |= (1 << lcd_rs_bit);
	lcd_en_dir |= (1 << lcd_en_bit);
	lcd_dir = 0xf0;
	lcd_port &= 0x0f;
	lcd_en_port |= (1 << lcd_en_bit);
	lcd_rs_port |= (1 << lcd_rs_bit);
}

void lcd_init(void)
{
	_delay_ms(50);
	lcd_acht_putcomm(0b00110000);  /* 8-Bit Modus */
	_delay_ms(5);
	lcd_acht_putcomm(0b00110000);
	_delay_ms(1);
	lcd_acht_putcomm(0b00110000);
	_delay_ms(1);

	lcd_acht_putcomm(0b00100000); /* 4-Bit Modus einschalten */
	_delay_ms(10);

	lcd_putcomm(0b00101000); /* 4-Bit Modus, 2 Zeilen, 5x7 Zeichen */
	_delay_ms(1);
	lcd_putcomm(0b00001000); /* ON/OFF control */
	_delay_ms(1);
	lcd_putcomm(0b00000001); /* Clear display */
	_delay_ms(10);
	lcd_putcomm(0b00000110); /* Cursor inkrement, Display fest */
	_delay_ms(1);
	lcd_putcomm(0b00001100); /* Display ein */
	_delay_ms(1);
}

void lcd_hex(unsigned char h)
{
	const char lcd_symbol[] = "0123456789abcdef";

	unsigned char b, c;

	
	b = h;

	
	b &= 0xf0;

	c = b >> 4;

	lcd_putchar(lcd_symbol[c]);
	lcd_putchar(lcd_symbol[h & 0x0f]);
}

void lcd_dec(int d)
{
	int i, v, f;

	i = 0;
	v = d;
	f = 0; 

	

	while (v >= 100)
	{
		v -= 100;
		i++;
	}
	

	lcd_putchar(i+48);


	i = 0;

	while (v >= 10)
	{
		v -= 10;
		i++;
	}

	lcd_putchar(i+48);

	i = 0;

	while (v > 0)
	{
		v--;
		i++;
	}
	lcd_putchar(i+48);
}	 

void lcd_cls(void)
{
	lcd_putcomm(0x01);
	_delay_ms(2);
}

void lcd_home(void)
{
	lcd_putcomm(0x02);
	_delay_ms(2);
}

void lcd_ddram(char adr)
{
	lcd_putcomm(0x80 | adr);
	_delay_us(50);
}

void lcd_int(unsigned int d)
{
	int i;
	unsigned int v;
	unsigned char f;

	i = 0;
	v = d;
	f = 0;

	while (v >= 10000)
	{
		v -= 10000;
		i++;
	}
	

	lcd_putchar(i+48);


	i = 0; 

	while (v >= 1000)
	{
		v -= 1000;
		i++;
	}
	

	lcd_putchar(i+48);


	i = 0;
	

	while (v >= 100)
	{
		v -= 100;
		i++;
	}
	

	lcd_putchar(i+48);


	i = 0;

	while (v >= 10)
	{
		v -= 10;
		i++;
	}

	
	lcd_putchar(i+48);


	i = 0;


	while (v > 0)
	{
		v--;
		i++;
	}

	lcd_putchar(i+48);
}	 

